﻿using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;
using System.Runtime.InteropServices;
using System.Windows.Forms;

namespace BeeDevelopment.WabbitemuFrontEnd {
	public partial class Main : Form {

		RevSoft.Wabbitemu Wabbit;
		Bitmap Screen;

		protected override bool IsInputKey(Keys keyData) {
			return true;
		}

		protected override void OnKeyDown(KeyEventArgs e) {
			Wabbit.KeyPress(e.KeyCode);
			base.OnKeyDown(e);
		}

		protected override void OnKeyUp(KeyEventArgs e) {
			Wabbit.KeyRelease(e.KeyCode);
			base.OnKeyUp(e);
		}

		public Main() {
			
			InitializeComponent();
			this.Text = Application.ProductName;

			this.Wabbit = new RevSoft.Wabbitemu();
			
			// Create a 128x64 pixel indexed bitmap to represent the screen.
			this.Screen = new Bitmap(128, 64, PixelFormat.Format8bppIndexed);
			
			// Set palette.
			var Darkest = Color.FromArgb(unchecked((int)0xFF11120E));
			var Lightest = Color.FromArgb(unchecked((int)0xFF9EAB88));
			var Palette = this.Screen.Palette;
			for (int i = 0; i < 255; ++i) {
				Palette.Entries[i] = Color.FromArgb(
					255,
					((Darkest.R - Lightest.R) * i) / 256 + Lightest.R,
					((Darkest.G - Lightest.G) * i) / 256 + Lightest.G,
					((Darkest.B - Lightest.B) * i) / 256 + Lightest.B
				);
			}
			this.Screen.Palette = Palette;

			this.SetStyle(ControlStyles.AllPaintingInWmPaint | ControlStyles.Opaque | ControlStyles.OptimizedDoubleBuffer, true);
		}

		protected override void OnPaint(PaintEventArgs e) {

			e.Graphics.Clear(Color.FromKnownColor(KnownColor.Control));
			e.Graphics.InterpolationMode = InterpolationMode.NearestNeighbor;
			e.Graphics.PixelOffsetMode = PixelOffsetMode.Half;

			byte[] Data = this.Wabbit.GetScreenData();
			
			var LockedScreen = this.Screen.LockBits(new Rectangle(0, 0, 128, 64), ImageLockMode.WriteOnly, PixelFormat.Format8bppIndexed);
			Marshal.Copy(Data, 0, LockedScreen.Scan0, Data.Length);
			this.Screen.UnlockBits(LockedScreen);
			
			e.Graphics.ResetTransform();
			e.Graphics.TranslateTransform(
					(this.ClientSize.Width - 96 * 2) / 2,
					(this.ClientSize.Height - 64 * 2) / 2
			);
			e.Graphics.DrawImage(this.Screen, new Rectangle(0, 0, 96 * 2, 64 * 2), new Rectangle(0, 0, 96, 64), GraphicsUnit.Pixel);

			base.OnPaint(e);
		}

		private void RefreshState_Tick(object sender, EventArgs e) {
			this.Invalidate();
		}

		private void Main_Click(object sender, EventArgs e) {
			//this.Wabbit.Reset(RevSoft.Wabbitemu.MemoryTypes.Both);
		}

	}
}
